@@ -1,4 +1,4 @@
-/* GrandPerspective, Version 2.5.4 
+/* GrandPerspective, Version 2.6.0 
  *   A utility for Mac OS X that graphically shows disk usage. 
  * Copyright (C) 2005-2020, Erwin Bonsma 
  * 
@@ -79,6 +79,11 @@ extern NSString  *EstimatedProgressKey;
 }
 
 - (void) windowDidLoad {
+  if (@available(macOS 10.11, *)) {
+    self->progressSummary.font = [NSFont monospacedDigitSystemFontOfSize: 0
+                                                                  weight: NSFontWeightRegular];
+  }
+
   [self updateProgressDetails: @""];
   [self updateProgressSummary: 0];
   
@@ -159,7 +164,7 @@ extern NSString  *EstimatedProgressKey;
 
 - (void) updateProgressSummary:(int)numProcessed {
   progressSummary.stringValue =
-    [NSString stringWithFormat: self.progressSummaryFormat, numProcessed];
+    [NSString localizedStringWithFormat: self.progressSummaryFormat, numProcessed];
 }
 
 - (void) updateProgressEstimate:(float)progressEstimate {
