@@ -1,4 +1,4 @@
-/* GrandPerspective, Version 2.5.4 
+/* GrandPerspective, Version 2.6.0 
  *   A utility for Mac OS X that graphically shows disk usage. 
  * Copyright (C) 2005-2020, Erwin Bonsma 
  * 
@@ -253,7 +253,8 @@ NSString  *FriendlySizeKey = @"friendlySize";
 
 
 - (BOOL) clearVisiblePath {
-  NSAssert(!visiblePathLocked, @"Cannot clear path when locked.");
+  // The below assertion is not valid anymore. Keyboard navigation can now change locked paths.
+  //NSAssert(!visiblePathLocked, @"Cannot clear path when locked.");
 
   NSUInteger  num = path.count - visibleTreeIndex - 1;
 
@@ -272,7 +273,8 @@ NSString  *FriendlySizeKey = @"friendlySize";
 }
 
 - (void) extendVisiblePath:(Item *)nextItem {
-  NSAssert(!visiblePathLocked, @"Cannot extend path when locked.");
+  // The below assertion is not valid anymore. Keyboard navigation can now change locked paths.
+  //NSAssert(!visiblePathLocked, @"Cannot extend path when locked.");
    
   [path addObject: nextItem]; 
 
