@@ -1,4 +1,4 @@
-/* GrandPerspective, Version 2.5.4 
+/* GrandPerspective, Version 2.6.0 
  *   A utility for Mac OS X that graphically shows disk usage. 
  * Copyright (C) 2005-2020, Erwin Bonsma 
  * 
@@ -38,7 +38,7 @@ extern NSString  *ColorMappingChangedEvent;
 @class ItemPathDrawer;
 @class ItemPathModelView;
 @class OverlayDrawer;
-@class SelectedItemLocator;
+@class ItemLocator;
 @protocol FileItemMappingScheme;
 
 @interface DirectoryView : NSView {
@@ -53,7 +53,7 @@ extern NSString  *ColorMappingChangedEvent;
 
   ItemPathDrawer  *pathDrawer;
   ItemPathModelView  *pathModelView;
-  SelectedItemLocator  *selectedItemLocator;
+  ItemLocator  *selectedItemLocator;
   
   // The current color mapping, which is being observed for any changes to the scheme.
   NSObject <FileItemMappingScheme>  *observedColorMapping;
