@@ -1,4 +1,4 @@
-/* GrandPerspective, Version 2.5.4 
+/* GrandPerspective, Version 2.6.0 
  *   A utility for Mac OS X that graphically shows disk usage. 
  * Copyright (C) 2005-2020, Erwin Bonsma 
  * 
@@ -30,7 +30,7 @@
 #import "ItemPathDrawer.h"
 #import "ItemPathModel.h"
 #import "ItemPathModelView.h"
-#import "SelectedItemLocator.h"
+#import "ItemLocator.h"
 
 #import "OverlayDrawer.h"
 
@@ -84,6 +84,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 - (void) colorMappingChanged:(NSNotification *)notification;
 
 - (void) updateSelectedItem:(NSPoint)point;
+- (void) moveSelectedItem:(DirectionEnum)direction;
 
 @end 
 
@@ -94,7 +95,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
   if (self = [super initWithFrame:frame]) {
     layoutBuilder = [[TreeLayoutBuilder alloc] init];
     pathDrawer = [[ItemPathDrawer alloc] init];
-    selectedItemLocator = [[SelectedItemLocator alloc] init];
+    selectedItemLocator = [[ItemLocator alloc] init];
 
     scrollWheelDelta = 0;
   }
@@ -159,7 +160,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
   [nc addObserver: self
          selector: @selector(visiblePathLockingChanged:)
              name: VisiblePathLockingChangedEvent
-           object: [pathModelView pathModel]];
+           object: pathModelView.pathModel];
 
   [nc addObserver: self
          selector: @selector(windowMainStatusChanged:)
@@ -188,15 +189,13 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 }
 
 - (FileItem *)treeInView {
-  return (showEntireVolume 
-          ? [pathModelView volumeTree]
-          : [pathModelView visibleTree]);
+  return showEntireVolume ? pathModelView.volumeTree : pathModelView.visibleTree;
 }
 
 
 - (NSRect) locationInViewForItemAtEndOfPath:(NSArray *)itemPath {
   return [selectedItemLocator locationForItemAtEndOfPath: itemPath
-                                          startingAtTree: [self treeInView]
+                                          startingAtTree: self.treeInView
                                       usingLayoutBuilder: layoutBuilder
                                                   bounds: self.bounds];
 }
@@ -218,15 +217,15 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 
 - (TreeDrawerSettings *)treeDrawerSettings {
-  DrawTaskExecutor  *drawTaskExecutor = (DrawTaskExecutor*)[drawTaskManager taskExecutor];
+  DrawTaskExecutor  *drawTaskExecutor = (DrawTaskExecutor*)drawTaskManager.taskExecutor;
 
-  return [drawTaskExecutor treeDrawerSettings];
+  return drawTaskExecutor.treeDrawerSettings;
 }
 
 - (void) setTreeDrawerSettings:(TreeDrawerSettings *)settings {
-  DrawTaskExecutor  *drawTaskExecutor = (DrawTaskExecutor*)[drawTaskManager taskExecutor];
+  DrawTaskExecutor  *drawTaskExecutor = (DrawTaskExecutor*)drawTaskManager.taskExecutor;
 
-  TreeDrawerSettings  *oldSettings = [drawTaskExecutor treeDrawerSettings];
+  TreeDrawerSettings  *oldSettings = drawTaskExecutor.treeDrawerSettings;
   if (settings != oldSettings) {
     [oldSettings retain];
 
@@ -265,9 +264,6 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
     overlayTest = [overlayTestVal retain];
 
     [self forceOverlayRedraw];
-
-    // Assumption is that this method is only invoked when the window is the main one.
-    [self enablePeriodicRedraw: (overlayTest != nil)];
   }
 }
 
@@ -290,12 +286,11 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 
 - (BOOL) canZoomIn {
-  return ( [[pathModelView pathModel] isVisiblePathLocked] 
-           && [pathModelView canMoveVisibleTreeDown] );
+  return pathModelView.pathModel.isVisiblePathLocked && pathModelView.canMoveVisibleTreeDown;
 }
 
 - (BOOL) canZoomOut {
-  return [pathModelView canMoveVisibleTreeUp];
+  return pathModelView.canMoveVisibleTreeUp;
 }
 
 
@@ -307,16 +302,16 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
   [pathModelView moveVisibleTreeUp];
   
   // Automatically lock path as well.
-  [[pathModelView pathModel] setVisiblePathLocking: YES];
+  [pathModelView.pathModel setVisiblePathLocking: YES];
 }
 
 
 - (BOOL) canMoveFocusUp {
-  return [pathModelView canMoveSelectionUp];
+  return pathModelView.canMoveSelectionUp;
 }
 
 - (BOOL) canMoveFocusDown {
-  return ! [pathModelView selectionSticksToEndPoint];
+  return !pathModelView.selectionSticksToEndPoint;
 }
 
 
@@ -325,7 +320,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 }
 
 - (void) moveFocusDown {
-  if ([pathModelView canMoveSelectionDown]) {
+  if (pathModelView.canMoveSelectionDown) {
     [pathModelView moveSelectionDown];
   }
   else {
@@ -335,7 +330,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 
 - (void) drawRect:(NSRect)rect {
-  if (pathModelView==nil) {
+  if (pathModelView == nil) {
     return;
   }
   
@@ -363,7 +358,6 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
   }
   
   if (treeImage != nil) {
-    //NSLog(@"Drawing image");
     [treeImage drawAtPoint: NSZeroPoint
                   fromRect: NSZeroRect
                  operation: NSCompositeCopy
@@ -377,7 +371,6 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
     }
 
     if (!treeImageIsScaled) {
-      //NSLog(@"Drawing path");
       if ([pathModelView isSelectedFileItemVisible]) {
         [pathDrawer drawVisiblePath: pathModelView
                      startingAtTree: self.treeInView
@@ -409,6 +402,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 - (BOOL)performKeyEquivalent:(NSEvent *)theEvent {
   int  flags = theEvent.modifierFlags & NSDeviceIndependentModifierFlagsMask;
   NSString  *chars = theEvent.characters;
+  unichar const  code = [chars characterAtIndex: 0];
   
   if ([chars isEqualToString: @"]"]) {
     if (flags == NSCommandKeyMask) {
@@ -455,7 +449,22 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
       return YES;
     }
   }
-  
+  else if (pathModelView.pathModel.isVisiblePathLocked) {
+    // Navigation via arrow keys is active when the path is locked (so that mouse movement does not
+    // interfere).
+    BOOL handled = YES;
+    switch (code) {
+      case NSUpArrowFunctionKey: [self moveSelectedItem: DirectionUp]; break;
+      case NSDownArrowFunctionKey: [self moveSelectedItem: DirectionDown]; break;
+      case NSRightArrowFunctionKey: [self moveSelectedItem: DirectionRight]; break;
+      case NSLeftArrowFunctionKey: [self moveSelectedItem: DirectionLeft]; break;
+      default: handled = NO;
+    }
+    if (handled) {
+      return YES;
+    }
+  }
+
   return NO;
 }
 
@@ -491,10 +500,9 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 
 - (void) mouseDown:(NSEvent *)theEvent {
-  ItemPathModel  *pathModel = [pathModelView pathModel];
+  ItemPathModel  *pathModel = pathModelView.pathModel;
 
-  if (self.window.acceptsMouseMovedEvents &&
-      [pathModel lastFileItem] == [pathModel visibleTree]) {
+  if (self.window.acceptsMouseMovedEvents && pathModel.lastFileItem == pathModel.visibleTree) {
     // Although the visible path is following the mouse, the visible path is empty. This can either
     // mean that the view only shows a single file item or, more likely, the view did not yet
     // receive the mouse moved events that are required to update the visible path because it was
@@ -503,7 +511,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
     // Force building (and drawing) of the visible path.
     [self mouseMoved: theEvent];
     
-    if ( [pathModel lastFileItem] != [pathModel visibleTree] ) {
+    if (pathModel.lastFileItem != pathModel.visibleTree) {
       // The path changed. Do not toggle the locking. This mouse click was used to make the view the
       // first responder, ensuring that the visible path is following the mouse pointer.
       return;
@@ -512,7 +520,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
   // Toggle the path locking.
 
-  BOOL  wasLocked = [pathModel isVisiblePathLocked];
+  BOOL  wasLocked = pathModel.isVisiblePathLocked;
   if (wasLocked) {
     // Unlock first, then build new path.
     [pathModel setVisiblePathLocking: NO];
@@ -524,7 +532,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
   if (!wasLocked) {
     // Now lock, after having updated path.
 
-    if ([pathModelView isSelectedFileItemVisible]) {
+    if (pathModelView.isSelectedFileItemVisible) {
       // Only lock the path if it contains the selected item, i.e. if the mouse click was inside the
       // visible tree.
       [pathModel setVisiblePathLocking: YES];
@@ -534,12 +542,11 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 
 - (void) mouseMoved:(NSEvent *)theEvent {
-  if ([[pathModelView pathModel] isVisiblePathLocked]) {
+  if (pathModelView.pathModel.isVisiblePathLocked) {
     // Ignore mouseMoved events when the item path is locked.
     //
     // Note: Although this view stops accepting mouse moved events when the path becomes locked,
-    // these may be generated later on anyway, requested by other components. In particular, mousing
-    // over the NSTextViews in the drawer triggers mouse moved events again.
+    // these may be generated later on anyway, requested by other components.
     return;
   }
   
@@ -549,8 +556,8 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
   }
   
   NSPoint  loc = self.window.mouseLocationOutsideOfEventStream;
-  // Note: not using the location returned by [theEvent locationInWindow] as this is incorrect after
-  // the drawer has been clicked on.
+  // Note: not using the location returned by [theEvent locationInWindow] as this is not fully
+  // accurate.
 
   NSPoint  mouseLoc = [self convertPoint: loc fromView: nil];
   BOOL isInside = [self mouse: mouseLoc inRect: self.bounds];
@@ -559,7 +566,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
     [self updateSelectedItem: mouseLoc];
   }
   else {
-    [[pathModelView pathModel] clearVisiblePath];
+    [pathModelView.pathModel clearVisiblePath];
   }
 }
 
@@ -729,7 +736,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 - (float) animatedOverlayStrength {
   return (self.window.mainWindow
-          ? 0.7 + 0.3 * sin([[NSDate date] timeIntervalSinceReferenceDate] * 3.1415)
+          ? 0.7 + 0.3 * sin([NSDate date].timeIntervalSinceReferenceDate * 3.1415)
           : 0.7);
 }
 
@@ -740,7 +747,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 - (void) enablePeriodicRedraw:(BOOL) enable {
   if (enable) {
     if (redrawTimer == nil) {
-      redrawTimer = [NSTimer scheduledTimerWithTimeInterval: 0.2f
+      redrawTimer = [NSTimer scheduledTimerWithTimeInterval: 0.1f
                                                      target: self
                                                    selector: @selector(refreshDisplay)
                                                    userInfo: nil
@@ -756,13 +763,13 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 }
 
 - (void) postColorPaletteChanged {
-  [[NSNotificationCenter defaultCenter]
-      postNotificationName: ColorPaletteChangedEvent object: self];
+  [[NSNotificationCenter defaultCenter] postNotificationName: ColorPaletteChangedEvent
+                                                      object: self];
 }
 
 - (void) postColorMappingChanged {
-  [[NSNotificationCenter defaultCenter]
-      postNotificationName: ColorMappingChangedEvent object: self];
+  [[NSNotificationCenter defaultCenter] postNotificationName: ColorMappingChangedEvent
+                                                      object: self];
 }
 
 
@@ -777,13 +784,11 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 }
 
 - (void) visiblePathLockingChanged:(NSNotification *)notification {
-  BOOL  locked = [[pathModelView pathModel] isVisiblePathLocked];
-  
   // Update the item path drawer directly. Although the drawer could also listen to the
   // notification, it seems better to do it like this. It keeps the item path drawer more general,
   // and as the item path drawer is tightly integrated with this view, there is no harm in updating
   // it directly.
-  [pathDrawer setHighlightPathEndPoint: locked];
+  [pathDrawer setHighlightPathEndPoint: pathModelView.pathModel.isVisiblePathLocked];
  
   [self updateAcceptMouseMovedEvents];
   
@@ -793,8 +798,9 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 - (void) windowMainStatusChanged:(NSNotification *)notification {
   [self updateAcceptMouseMovedEvents];
 
-  // Only when the window is the main one and there is an overlay active, animate the overlay.
-  [self enablePeriodicRedraw: (self.window.mainWindow && overlayTest != nil)];
+  // Only when the window is the main one enable periodic redraw. This takes care of the overlay
+  // animation as well as the selected item highlight animation.
+  [self enablePeriodicRedraw: self.window.mainWindow];
 }
 
 - (void) windowKeyStatusChanged:(NSNotification *)notification {
@@ -802,11 +808,10 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 }
 
 - (void) updateAcceptMouseMovedEvents {
-  BOOL  letPathFollowMouse = 
-    ( ![[pathModelView pathModel] isVisiblePathLocked] 
-      && self.window.mainWindow
-      && self.window.keyWindow );
-      
+  BOOL  letPathFollowMouse = !pathModelView.pathModel.isVisiblePathLocked
+                              && self.window.mainWindow
+                              && self.window.keyWindow;
+
   self.window.acceptsMouseMovedEvents = letPathFollowMouse;
 
   if (letPathFollowMouse) {
@@ -819,7 +824,7 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 - (void) observeColorMapping {
   TreeDrawerSettings  *treeDrawerSettings = [self treeDrawerSettings];
   NSObject <FileItemMappingScheme>  *colorMapping = 
-    [[treeDrawerSettings colorMapper] fileItemMappingScheme];
+    treeDrawerSettings.colorMapper.fileItemMappingScheme;
     
   if (colorMapping != observedColorMapping) {
     NSNotificationCenter  *nc = [NSNotificationCenter defaultCenter];
@@ -842,9 +847,9 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 - (void) colorMappingChanged:(NSNotification *) notification {
   // Replace the mapper that is used by a new one (still from the same scheme)
   NSObject <FileItemMapping>  *newMapping =
-    [observedColorMapping fileItemMappingForTree: [pathModelView scanTree]];
+    [observedColorMapping fileItemMappingForTree: pathModelView.scanTree];
 
-  [self setTreeDrawerSettings: [[self treeDrawerSettings] copyWithColorMapper: newMapping]];
+  [self setTreeDrawerSettings: [self.treeDrawerSettings copyWithColorMapper: newMapping]];
 
   [self postColorMappingChanged]; 
 }
@@ -852,10 +857,17 @@ NSString  *ColorMappingChangedEvent = @"colorMappingChanged";
 
 - (void) updateSelectedItem: (NSPoint) point {
   [pathModelView selectItemAtPoint: point 
-                    startingAtTree: [self treeInView]
+                    startingAtTree: self.treeInView
                 usingLayoutBuilder: layoutBuilder
                             bounds: self.bounds];
   // Redrawing in response to any changes will happen when the change notification is received.
 }
 
+- (void) moveSelectedItem: (DirectionEnum) direction {
+  [pathModelView moveSelectedItem: direction
+                  startingAtTree: self.treeInView
+              usingLayoutBuilder: layoutBuilder
+                          bounds: self.bounds];
+}
+
 @end // @implementation DirectoryView (PrivateMethods)
