@@ -1,4 +1,4 @@
-/* GrandPerspective, Version 2.5.4 
+/* GrandPerspective, Version 2.6.0 
  *   A utility for Mac OS X that graphically shows disk usage. 
  * Copyright (C) 2005-2020, Erwin Bonsma 
  * 
@@ -67,7 +67,8 @@
   // clearly visible, even when some/all borders align with the view's bounds, given that the bounds
   // are not expanded.
   if (prevRect.size.width > 0 && (! NSEqualRects(prevRect, outerRect) || highlightPathEndPoint)) {
-    [[NSColor selectedControlColor] set];
+    float fraction = 0.8 + 0.2 * sin([NSDate date].timeIntervalSinceReferenceDate * 3.1415);
+    [[NSColor.selectedControlColor highlightWithLevel: fraction] set];
 
     NSBezierPath  *path = [NSBezierPath bezierPathWithRect: prevRect];
     
@@ -104,7 +105,8 @@
         // okay, because it is not the endpoint. The endpoint definitely needs to be shown to
         // provide the user with visual feedback needed to move the focus and to lock and unlock the
         // selection)
-        [[NSColor secondarySelectedControlColor] set];
+
+        [NSColor.selectedControlColor set];
 
         NSRect  drawRect = NSMakeRect(prevRect.origin.x - 1,
                                       prevRect.origin.y - 1,
