@@ -1,4 +1,4 @@
-/* GrandPerspective, Version 2.5.4 
+/* GrandPerspective, Version 2.6.0 
  *   A utility for Mac OS X that graphically shows disk usage. 
  * Copyright (C) 2005-2020, Erwin Bonsma 
  * 
@@ -81,7 +81,7 @@
 }
 
 - (FileItemTest *)maskTest {
-  return [treeGuide fileItemTest];
+  return treeGuide.fileItemTest;
 }
 
 
@@ -90,7 +90,7 @@
 }
 
 - (BOOL) showPackageContents {
-  return ! [treeGuide packagesAsFiles];
+  return !treeGuide.packagesAsFiles;
 }
 
 
@@ -123,7 +123,7 @@
 
 - (void)drawFile:(PlainFileItem *)fileItem atRect:(NSRect) rect depth:(int) depth {
   NSUInteger  colorIndex = [colorMapper hashForFileItem: fileItem atDepth: depth];
-  if ( [colorMapper canProvideLegend] ) {
+  if (colorMapper.canProvideLegend) {
     colorIndex = MIN(colorIndex, rectangleDrawer.numGradientColors - 1);
   }
   else {
